@@ -1,6 +1,6 @@
 /*
  * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2009 Breach Security, Inc. (http://www.breach.com/)
+ * Copyright (c) 2004-2010 Breach Security, Inc. (http://www.breach.com/)
  *
  * This product is released under the terms of the General Public Licence,
  * version 2 (GPLv2). Please refer to the file LICENSE (included with this
@@ -34,7 +34,8 @@
 /**
  * Creates a fresh directory configuration.
  */
-void *create_directory_config(apr_pool_t *mp, char *path) {
+void *create_directory_config(apr_pool_t *mp, char *path)
+{
     directory_config *dcfg = (directory_config *)apr_pcalloc(mp, sizeof(directory_config));
     if (dcfg == NULL) return NULL;
 
@@ -88,6 +89,7 @@ void *create_directory_config(apr_pool_t *mp, char *path) {
     dcfg->upload_keep_files = NOT_SET;
     dcfg->upload_validates_files = NOT_SET;
     dcfg->upload_filemode = NOT_SET;
+    dcfg->upload_file_limit = NOT_SET;
 
     /* These are only used during the configuration process. */
     dcfg->tmp_chain_starter = NULL;
@@ -130,8 +132,10 @@ void *create_directory_config(apr_pool_t *mp, char *path) {
  * Copies rules between one phase of two configuration contexts,
  * taking exceptions into account.
  */
-static void copy_rules_phase(apr_pool_t *mp, apr_array_header_t *parent_phase_arr,
-    apr_array_header_t *child_phase_arr, apr_array_header_t *exceptions_arr)
+static void copy_rules_phase(apr_pool_t *mp,
+                             apr_array_header_t *parent_phase_arr,
+                             apr_array_header_t *child_phase_arr,
+                             apr_array_header_t *exceptions_arr)
 {
     rule_exception **exceptions;
     msre_rule **rules;
@@ -199,8 +203,9 @@ static void copy_rules_phase(apr_pool_t *mp, apr_array_header_t *parent_phase_ar
  * Copies rules between two configuration contexts,
  * taking exceptions into account.
  */
-static int copy_rules(apr_pool_t *mp, msre_ruleset *parent_ruleset, msre_ruleset *child_ruleset,
-    apr_array_header_t *exceptions_arr)
+static int copy_rules(apr_pool_t *mp, msre_ruleset *parent_ruleset,
+                      msre_ruleset *child_ruleset,
+                      apr_array_header_t *exceptions_arr)
 {
     copy_rules_phase(mp, parent_ruleset->phase_request_headers,
         child_ruleset->phase_request_headers, exceptions_arr);
@@ -219,7 +224,8 @@ static int copy_rules(apr_pool_t *mp, msre_ruleset *parent_ruleset, msre_ruleset
 /**
  * Merges two directory configurations.
  */
-void *merge_directory_configs(apr_pool_t *mp, void *_parent, void *_child) {
+void *merge_directory_configs(apr_pool_t *mp, void *_parent, void *_child)
+{
     directory_config *parent = (directory_config *)_parent;
     directory_config *child = (directory_config *)_child;
     directory_config *merged = create_directory_config(mp, NULL);
@@ -427,6 +433,8 @@ void *merge_directory_configs(apr_pool_t *mp, void *_parent, void *_child) {
         ? parent->upload_validates_files : child->upload_validates_files);
     merged->upload_filemode = (child->upload_filemode == NOT_SET
         ? parent->upload_filemode : child->upload_filemode);
+    merged->upload_file_limit = (child->upload_file_limit == NOT_SET
+        ? parent->upload_file_limit : child->upload_file_limit);
 
     /* Misc */
     merged->data_dir = (child->data_dir == NOT_SET_P
@@ -484,7 +492,8 @@ void *merge_directory_configs(apr_pool_t *mp, void *_parent, void *_child) {
  * the configuration phase. It can only be called on copies of those
  * (created fresh for every transaction).
  */
-void init_directory_config(directory_config *dcfg) {
+void init_directory_config(directory_config *dcfg)
+{
     if (dcfg == NULL) return;
 
     if (dcfg->is_enabled == NOT_SET) dcfg->is_enabled = 0;
@@ -534,7 +543,8 @@ void init_directory_config(directory_config *dcfg) {
     if (dcfg->upload_dir == NOT_SET_P) dcfg->upload_dir = NULL;
     if (dcfg->upload_keep_files == NOT_SET) dcfg->upload_keep_files = KEEP_FILES_OFF;
     if (dcfg->upload_validates_files == NOT_SET) dcfg->upload_validates_files = 0;
-    if (dcfg->upload_filemode == NOT_SET) dcfg->upload_filemode = mode2fileperms(0600);
+    if (dcfg->upload_filemode == NOT_SET) dcfg->upload_filemode = 0600;
+    if (dcfg->upload_file_limit == NOT_SET) dcfg->upload_file_limit = 100;
 
     /* Misc */
     if (dcfg->data_dir == NOT_SET_P) dcfg->data_dir = NULL;
@@ -562,13 +572,14 @@ void init_directory_config(directory_config *dcfg) {
     if (dcfg->cache_trans_maxitems == (apr_size_t)NOT_SET) dcfg->cache_trans_maxitems = 512;
 
     if (dcfg->request_encoding == NOT_SET_P) dcfg->request_encoding = NULL;
+
 }
 
 /**
  *
  */
 static const char *add_rule(cmd_parms *cmd, directory_config *dcfg, int type,
-    const char *p1, const char *p2, const char *p3)
+                            const char *p1, const char *p2, const char *p3)
 {
     char *my_error_msg = NULL;
     msre_rule *rule = NULL;
@@ -755,8 +766,8 @@ static const char *add_rule(cmd_parms *cmd, directory_config *dcfg, int type,
 /**
  *
  */
-static const char *add_marker(cmd_parms *cmd, directory_config *dcfg, const char *p1,
-    const char *p2, const char *p3)
+static const char *add_marker(cmd_parms *cmd, directory_config *dcfg,
+                              const char *p1, const char *p2, const char *p3)
 {
     char *my_error_msg = NULL;
     msre_rule *rule = NULL;
@@ -808,7 +819,7 @@ static const char *add_marker(cmd_parms *cmd, directory_config *dcfg, const char
  *
  */
 static const char *update_rule_action(cmd_parms *cmd, directory_config *dcfg,
-    const char *p1, const char *p2)
+                                      const char *p1, const char *p2)
 {
     char *my_error_msg = NULL;
     msre_rule *rule = NULL;
@@ -893,17 +904,21 @@ static const char *update_rule_action(cmd_parms *cmd, directory_config *dcfg,
 
 /* -- Configuration directives -- */
 
-static const char *cmd_action(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_action(cmd_parms *cmd, void *_dcfg, const char *p1)
+{
     return add_rule(cmd, (directory_config *)_dcfg, RULE_TYPE_ACTION, SECACTION_TARGETS, SECACTION_ARGS, p1);
 }
 
-static const char *cmd_marker(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_marker(cmd_parms *cmd, void *_dcfg, const char *p1)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
     const char *action = apr_pstrcat(dcfg->mp, SECMARKER_BASE_ACTIONS, p1, NULL);
     return add_marker(cmd, (directory_config *)_dcfg, SECMARKER_TARGETS, SECMARKER_ARGS, action);
 }
 
-static const char *cmd_argument_separator(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_argument_separator(cmd_parms *cmd, void *_dcfg,
+                                          const char *p1)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
 
     if (strlen(p1) != 1) {
@@ -915,7 +930,8 @@ static const char *cmd_argument_separator(cmd_parms *cmd, void *_dcfg, const cha
     return NULL;
 }
 
-static const char *cmd_audit_engine(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_audit_engine(cmd_parms *cmd, void *_dcfg, const char *p1)
+{
     directory_config *dcfg = _dcfg;
 
     if (strcasecmp(p1, "On") == 0) dcfg->auditlog_flag = AUDITLOG_ON;
@@ -930,7 +946,8 @@ static const char *cmd_audit_engine(cmd_parms *cmd, void *_dcfg, const char *p1)
     return NULL;
 }
 
-static const char *cmd_audit_log(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_audit_log(cmd_parms *cmd, void *_dcfg, const char *p1)
+{
     directory_config *dcfg = _dcfg;
 
     dcfg->auditlog_name = (char *)p1;
@@ -963,7 +980,8 @@ static const char *cmd_audit_log(cmd_parms *cmd, void *_dcfg, const char *p1) {
     return NULL;
 }
 
-static const char *cmd_audit_log2(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_audit_log2(cmd_parms *cmd, void *_dcfg, const char *p1)
+{
     directory_config *dcfg = _dcfg;
 
     if (dcfg->auditlog_name == NOT_SET_P) {
@@ -1000,7 +1018,9 @@ static const char *cmd_audit_log2(cmd_parms *cmd, void *_dcfg, const char *p1) {
     return NULL;
 }
 
-static const char *cmd_audit_log_parts(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_audit_log_parts(cmd_parms *cmd, void *_dcfg,
+                                       const char *p1)
+{
     directory_config *dcfg = _dcfg;
 
     if (is_valid_parts_specification((char *)p1) != 1) {
@@ -1011,7 +1031,9 @@ static const char *cmd_audit_log_parts(cmd_parms *cmd, void *_dcfg, const char *
     return NULL;
 }
 
-static const char *cmd_audit_log_relevant_status(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_audit_log_relevant_status(cmd_parms *cmd, void *_dcfg,
+                                                 const char *p1)
+{
     directory_config *dcfg = _dcfg;
 
     dcfg->auditlog_relevant_regex = msc_pregcomp(cmd->pool, p1, PCRE_DOTALL, NULL, NULL);
@@ -1022,7 +1044,9 @@ static const char *cmd_audit_log_relevant_status(cmd_parms *cmd, void *_dcfg, co
     return NULL;
 }
 
-static const char *cmd_audit_log_type(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_audit_log_type(cmd_parms *cmd, void *_dcfg,
+                                      const char *p1)
+{
     directory_config *dcfg = _dcfg;
 
     if (strcasecmp(p1, "Serial") == 0) dcfg->auditlog_type = AUDITLOG_SERIAL;
@@ -1035,7 +1059,9 @@ static const char *cmd_audit_log_type(cmd_parms *cmd, void *_dcfg, const char *p
     return NULL;
 }
 
-static const char *cmd_audit_log_dirmode(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_audit_log_dirmode(cmd_parms *cmd, void *_dcfg,
+                                         const char *p1)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
 
     if (dcfg == NULL) return NULL;
@@ -1055,7 +1081,9 @@ static const char *cmd_audit_log_dirmode(cmd_parms *cmd, void *_dcfg, const char
     return NULL;
 }
 
-static const char *cmd_audit_log_filemode(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_audit_log_filemode(cmd_parms *cmd, void *_dcfg,
+                                          const char *p1)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
 
     if (dcfg == NULL) return NULL;
@@ -1075,7 +1103,9 @@ static const char *cmd_audit_log_filemode(cmd_parms *cmd, void *_dcfg, const cha
     return NULL;
 }
 
-static const char *cmd_audit_log_storage_dir(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_audit_log_storage_dir(cmd_parms *cmd, void *_dcfg,
+                                             const char *p1)
+{
     directory_config *dcfg = _dcfg;
 
     dcfg->auditlog_storage_dir = ap_server_root_relative(cmd->pool, p1);
@@ -1083,7 +1113,9 @@ static const char *cmd_audit_log_storage_dir(cmd_parms *cmd, void *_dcfg, const
     return NULL;
 }
 
-static const char *cmd_cookie_format(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_cookie_format(cmd_parms *cmd, void *_dcfg,
+                                     const char *p1)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
 
     if (strcmp(p1, "0") == 0) dcfg->cookie_format = COOKIES_V0;
@@ -1096,7 +1128,8 @@ static const char *cmd_cookie_format(cmd_parms *cmd, void *_dcfg, const char *p1
     return NULL;
 }
 
-static const char *cmd_chroot_dir(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_chroot_dir(cmd_parms *cmd, void *_dcfg, const char *p1)
+{
     char cwd[1025] = "";
 
     if (cmd->server->is_virtual) {
@@ -1125,7 +1158,9 @@ static const char *cmd_chroot_dir(cmd_parms *cmd, void *_dcfg, const char *p1) {
 /**
  * Adds component signature to the list of signatures kept in configuration.
  */
-static const char *cmd_component_signature(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_component_signature(cmd_parms *cmd, void *_dcfg,
+                                           const char *p1)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
 
     /* ENH Enforce "Name/VersionX.Y.Z (comment)" format. */
@@ -1134,14 +1169,16 @@ static const char *cmd_component_signature(cmd_parms *cmd, void *_dcfg, const ch
     return NULL;
 }
 
-static const char *cmd_content_injection(cmd_parms *cmd, void *_dcfg, int flag) {
+static const char *cmd_content_injection(cmd_parms *cmd, void *_dcfg, int flag)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
     if (dcfg == NULL) return NULL;
     dcfg->content_injection_enabled = flag;
     return NULL;
 }
 
-static const char *cmd_data_dir(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_data_dir(cmd_parms *cmd, void *_dcfg, const char *p1)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
 
     if (cmd->server->is_virtual) {
@@ -1153,7 +1190,8 @@ static const char *cmd_data_dir(cmd_parms *cmd, void *_dcfg, const char *p1) {
     return NULL;
 }
 
-static const char *cmd_debug_log(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_debug_log(cmd_parms *cmd, void *_dcfg, const char *p1)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
     apr_status_t rc;
 
@@ -1171,7 +1209,9 @@ static const char *cmd_debug_log(cmd_parms *cmd, void *_dcfg, const char *p1) {
     return NULL;
 }
 
-static const char *cmd_debug_log_level(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_debug_log_level(cmd_parms *cmd, void *_dcfg,
+                                       const char *p1)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
 
     dcfg->debuglog_level = atoi(p1);
@@ -1180,7 +1220,9 @@ static const char *cmd_debug_log_level(cmd_parms *cmd, void *_dcfg, const char *
     return apr_psprintf(cmd->pool, "ModSecurity: Invalid value for SecDebugLogLevel: %s", p1);
 }
 
-static const char *cmd_default_action(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_default_action(cmd_parms *cmd, void *_dcfg,
+                                      const char *p1)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
     extern msc_engine *modsecurity;
     char *my_error_msg = NULL;
@@ -1244,7 +1286,9 @@ static const char *cmd_default_action(cmd_parms *cmd, void *_dcfg, const char *p
     return NULL;
 }
 
-static const char *cmd_guardian_log(cmd_parms *cmd, void *_dcfg, const char *p1, const char *p2) {
+static const char *cmd_guardian_log(cmd_parms *cmd, void *_dcfg,
+                                    const char *p1, const char *p2)
+{
     extern char *guardianlog_name;
     extern apr_file_t *guardianlog_fd;
     extern char *guardianlog_condition;
@@ -1293,7 +1337,9 @@ static const char *cmd_guardian_log(cmd_parms *cmd, void *_dcfg, const char *p1,
     return NULL;
 }
 
-static const char *cmd_request_body_inmemory_limit(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_request_body_inmemory_limit(cmd_parms *cmd, void *_dcfg,
+                                                   const char *p1)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
     long int limit;
 
@@ -1309,7 +1355,9 @@ static const char *cmd_request_body_inmemory_limit(cmd_parms *cmd, void *_dcfg,
     return NULL;
 }
 
-static const char *cmd_request_body_limit(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_request_body_limit(cmd_parms *cmd, void *_dcfg,
+                                          const char *p1)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
     long int limit;
 
@@ -1325,7 +1373,9 @@ static const char *cmd_request_body_limit(cmd_parms *cmd, void *_dcfg, const cha
     return NULL;
 }
 
-static const char *cmd_request_body_no_files_limit(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_request_body_no_files_limit(cmd_parms *cmd, void *_dcfg,
+                                                   const char *p1)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
     long int limit;
 
@@ -1341,7 +1391,9 @@ static const char *cmd_request_body_no_files_limit(cmd_parms *cmd, void *_dcfg,
     return NULL;
 }
 
-static const char *cmd_request_body_access(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_request_body_access(cmd_parms *cmd, void *_dcfg,
+                                           const char *p1)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
     if (dcfg == NULL) return NULL;
 
@@ -1354,7 +1406,9 @@ static const char *cmd_request_body_access(cmd_parms *cmd, void *_dcfg, const ch
     return NULL;
 }
 
-static const char *cmd_request_encoding(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_request_encoding(cmd_parms *cmd, void *_dcfg,
+                                        const char *p1)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
     if (dcfg == NULL) return NULL;
 
@@ -1365,7 +1419,9 @@ static const char *cmd_request_encoding(cmd_parms *cmd, void *_dcfg, const char
     return NULL;
 }
 
-static const char *cmd_response_body_access(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_response_body_access(cmd_parms *cmd, void *_dcfg,
+                                            const char *p1)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
     if (dcfg == NULL) return NULL;
 
@@ -1378,7 +1434,9 @@ static const char *cmd_response_body_access(cmd_parms *cmd, void *_dcfg, const c
     return NULL;
 }
 
-static const char *cmd_response_body_limit(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_response_body_limit(cmd_parms *cmd, void *_dcfg,
+                                           const char *p1)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
     long int limit;
 
@@ -1396,7 +1454,9 @@ static const char *cmd_response_body_limit(cmd_parms *cmd, void *_dcfg, const ch
     return NULL;
 }
 
-static const char *cmd_response_body_limit_action(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_response_body_limit_action(cmd_parms *cmd, void *_dcfg,
+                                                  const char *p1)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
     if (dcfg == NULL) return NULL;
 
@@ -1409,7 +1469,9 @@ static const char *cmd_response_body_limit_action(cmd_parms *cmd, void *_dcfg, c
     return NULL;
 }
 
-static const char *cmd_response_body_mime_type(cmd_parms *cmd, void *_dcfg, const char *_p1) {
+static const char *cmd_response_body_mime_type(cmd_parms *cmd, void *_dcfg,
+                                               const char *_p1)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
     char *p1 = apr_pstrdup(cmd->pool, _p1);
 
@@ -1425,7 +1487,9 @@ static const char *cmd_response_body_mime_type(cmd_parms *cmd, void *_dcfg, cons
     return NULL;
 }
 
-static const char *cmd_response_body_mime_types_clear(cmd_parms *cmd, void *_dcfg) {
+static const char *cmd_response_body_mime_types_clear(cmd_parms *cmd,
+                                                      void *_dcfg)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
     if (dcfg == NULL) return NULL;
 
@@ -1438,13 +1502,14 @@ static const char *cmd_response_body_mime_types_clear(cmd_parms *cmd, void *_dcf
     return NULL;
 }
 
-static const char *cmd_rule(cmd_parms *cmd, void *_dcfg, const char *p1,
-    const char *p2, const char *p3)
+static const char *cmd_rule(cmd_parms *cmd, void *_dcfg,
+                            const char *p1, const char *p2, const char *p3)
 {
     return add_rule(cmd, (directory_config *)_dcfg, RULE_TYPE_NORMAL, p1, p2, p3);
 }
 
-static const char *cmd_rule_engine(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_rule_engine(cmd_parms *cmd, void *_dcfg, const char *p1)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
     if (dcfg == NULL) return NULL;
 
@@ -1459,43 +1524,16 @@ static const char *cmd_rule_engine(cmd_parms *cmd, void *_dcfg, const char *p1)
     return NULL;
 }
 
-/*
-static const char *cmd_rule_import_by_id(cmd_parms *cmd, void *_dcfg, const char *p1) {
-    directory_config *dcfg = (directory_config *)_dcfg;
-    rule_exception *re = apr_pcalloc(cmd->pool, sizeof(rule_exception));
-    if (dcfg == NULL) return NULL;
-
-    re->type = RULE_EXCEPTION_IMPORT_ID;
-    // TODO verify p1
-    re->param = p1;
-       *(rule_exception **)apr_array_push(dcfg->rule_exceptions) = re;
-
-    return NULL;
-}
-
-static const char *cmd_rule_import_by_msg(cmd_parms *cmd, void *_dcfg, const char *p1) {
-    directory_config *dcfg = (directory_config *)_dcfg;
-    rule_exception *re = apr_pcalloc(cmd->pool, sizeof(rule_exception));
-    if (dcfg == NULL) return NULL;
-
-    re->type = RULE_EXCEPTION_IMPORT_MSG;
-    // TODO verify p1
-    re->param = p1;
-    *(rule_exception **)apr_array_push(dcfg->rule_exceptions) = re;
-
-    return NULL;
-}
-*/
-
-static const char *cmd_rule_inheritance(cmd_parms *cmd, void *_dcfg, int flag) {
+static const char *cmd_rule_inheritance(cmd_parms *cmd, void *_dcfg, int flag)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
     if (dcfg == NULL) return NULL;
     dcfg->rule_inheritance = flag;
     return NULL;
 }
 
-static const char *cmd_rule_script(cmd_parms *cmd, void *_dcfg, const char *p1,
-    const char *p2)
+static const char *cmd_rule_script(cmd_parms *cmd, void *_dcfg,
+                                   const char *p1, const char *p2)
 {
     #if defined(WITH_LUA)
     const char *filename = resolve_relative_path(cmd->pool, cmd->directive->filename, p1);
@@ -1506,7 +1544,9 @@ static const char *cmd_rule_script(cmd_parms *cmd, void *_dcfg, const char *p1,
     #endif
 }
 
-static const char *cmd_rule_remove_by_id(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_rule_remove_by_id(cmd_parms *cmd, void *_dcfg,
+                                         const char *p1)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
     rule_exception *re = apr_pcalloc(cmd->pool, sizeof(rule_exception));
     if (dcfg == NULL) return NULL;
@@ -1521,7 +1561,9 @@ static const char *cmd_rule_remove_by_id(cmd_parms *cmd, void *_dcfg, const char
     return NULL;
 }
 
-static const char *cmd_rule_remove_by_msg(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_rule_remove_by_msg(cmd_parms *cmd, void *_dcfg,
+                                          const char *p1)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
     rule_exception *re = apr_pcalloc(cmd->pool, sizeof(rule_exception));
     if (dcfg == NULL) return NULL;
@@ -1545,12 +1587,14 @@ static const char *cmd_rule_remove_by_msg(cmd_parms *cmd, void *_dcfg, const cha
 }
 
 static const char *cmd_rule_update_action_by_id(cmd_parms *cmd, void *_dcfg,
-    const char *p1, const char *p2)
+                                                const char *p1, const char *p2)
 {
     return update_rule_action(cmd, (directory_config *)_dcfg, p1, p2);
 }
 
-static const char *cmd_server_signature(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_server_signature(cmd_parms *cmd, void *_dcfg,
+                                        const char *p1)
+{
     if (cmd->server->is_virtual) {
         return "ModSecurity: SecServerSignature not allowed in VirtualHost";
     }
@@ -1558,7 +1602,8 @@ static const char *cmd_server_signature(cmd_parms *cmd, void *_dcfg, const char
     return NULL;
 }
 
-static const char *cmd_tmp_dir(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_tmp_dir(cmd_parms *cmd, void *_dcfg, const char *p1)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
 
     if (dcfg == NULL) return NULL;
@@ -1569,7 +1614,8 @@ static const char *cmd_tmp_dir(cmd_parms *cmd, void *_dcfg, const char *p1) {
     return NULL;
 }
 
-static const char *cmd_upload_dir(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_upload_dir(cmd_parms *cmd, void *_dcfg, const char *p1)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
 
     if (dcfg == NULL) return NULL;
@@ -1580,7 +1626,26 @@ static const char *cmd_upload_dir(cmd_parms *cmd, void *_dcfg, const char *p1) {
     return NULL;
 }
 
-static const char *cmd_upload_filemode(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_upload_file_limit(cmd_parms *cmd, void *_dcfg,
+                                         const char *p1)
+{
+    directory_config *dcfg = (directory_config *)_dcfg;
+
+    if (dcfg == NULL) return NULL;
+
+    if (strcasecmp(p1, "default") == 0) {
+        dcfg->upload_file_limit = NOT_SET;
+    }
+    else {
+        dcfg->upload_file_limit = atoi(p1);
+    }
+
+    return NULL;
+}
+
+static const char *cmd_upload_filemode(cmd_parms *cmd, void *_dcfg,
+                                       const char *p1)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
 
     if (dcfg == NULL) return NULL;
@@ -1600,7 +1665,9 @@ static const char *cmd_upload_filemode(cmd_parms *cmd, void *_dcfg, const char *
     return NULL;
 }
 
-static const char *cmd_upload_keep_files(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_upload_keep_files(cmd_parms *cmd, void *_dcfg,
+                                         const char *p1)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
 
     if (dcfg == NULL) return NULL;
@@ -1620,7 +1687,8 @@ static const char *cmd_upload_keep_files(cmd_parms *cmd, void *_dcfg, const char
     return NULL;
 }
 
-static const char *cmd_web_app_id(cmd_parms *cmd, void *_dcfg, const char *p1) {
+static const char *cmd_web_app_id(cmd_parms *cmd, void *_dcfg, const char *p1)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
 
     /* ENH enforce format (letters, digits, ., _, -) */
@@ -1629,9 +1697,53 @@ static const char *cmd_web_app_id(cmd_parms *cmd, void *_dcfg, const char *p1) {
     return NULL;
 }
 
+
+/* PCRE Limits */
+
+static const char *cmd_pcre_match_limit(cmd_parms *cmd,
+                                        void *_dcfg, const char *p1)
+{
+    long val;
+
+    if (cmd->server->is_virtual) {
+        return "ModSecurity: SecPcreMatchLimit not allowed in VirtualHost";
+    }
+
+    val = atol(p1);
+    if (val <= 0) {
+        return apr_psprintf(cmd->pool, "ModSecurity: Invalid setting for "
+                                       "SecPcreMatchLimit: %s", p1);
+    }
+    msc_pcre_match_limit = (unsigned long int)val;
+
+    return NULL;
+}
+
+static const char *cmd_pcre_match_limit_recursion(cmd_parms *cmd,
+                                        void *_dcfg, const char *p1)
+{
+    long val;
+
+    if (cmd->server->is_virtual) {
+        return "ModSecurity: SecPcreMatchLimitRecursion not allowed in VirtualHost";
+    }
+
+    val = atol(p1);
+    if (val <= 0) {
+        return apr_psprintf(cmd->pool, "ModSecurity: Invalid setting for "
+                                       "SecPcreMatchLimitRecursion: %s", p1);
+    }
+    msc_pcre_match_limit_recursion = (unsigned long int)val;
+
+    return NULL;
+}
+
+
+
 /* -- PDF Protection configuration -- */
 
-static const char *cmd_pdf_protect(cmd_parms *cmd, void *_dcfg, int flag) {
+static const char *cmd_pdf_protect(cmd_parms *cmd, void *_dcfg, int flag)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
     if (dcfg == NULL) return NULL;
 
@@ -1641,7 +1753,7 @@ static const char *cmd_pdf_protect(cmd_parms *cmd, void *_dcfg, int flag) {
 }
 
 static const char *cmd_pdf_protect_secret(cmd_parms *cmd, void *_dcfg,
-    const char *p1)
+                                          const char *p1)
 {
     directory_config *dcfg = (directory_config *)_dcfg;
     if (dcfg == NULL) return NULL;
@@ -1652,7 +1764,7 @@ static const char *cmd_pdf_protect_secret(cmd_parms *cmd, void *_dcfg,
 }
 
 static const char *cmd_pdf_protect_timeout(cmd_parms *cmd, void *_dcfg,
-    const char *p1)
+                                           const char *p1)
 {
     directory_config *dcfg = (directory_config *)_dcfg;
     if (dcfg == NULL) return NULL;
@@ -1663,7 +1775,7 @@ static const char *cmd_pdf_protect_timeout(cmd_parms *cmd, void *_dcfg,
 }
 
 static const char *cmd_pdf_protect_token_name(cmd_parms *cmd, void *_dcfg,
-    const char *p1)
+                                              const char *p1)
 {
     directory_config *dcfg = (directory_config *)_dcfg;
     if (dcfg == NULL) return NULL;
@@ -1673,8 +1785,8 @@ static const char *cmd_pdf_protect_token_name(cmd_parms *cmd, void *_dcfg,
     return NULL;
 }
 
-static const char *cmd_pdf_protect_intercept_get_only(cmd_parms *cmd, void *_dcfg,
-    int flag)
+static const char *cmd_pdf_protect_intercept_get_only(cmd_parms *cmd,
+                                                      void *_dcfg, int flag)
 {
     directory_config *dcfg = (directory_config *)_dcfg;
     if (dcfg == NULL) return NULL;
@@ -1685,7 +1797,7 @@ static const char *cmd_pdf_protect_intercept_get_only(cmd_parms *cmd, void *_dcf
 }
 
 static const char *cmd_pdf_protect_method(cmd_parms *cmd, void *_dcfg,
-    const char *p1)
+                                          const char *p1)
 {
     directory_config *dcfg = (directory_config *)_dcfg;
     if (dcfg == NULL) return NULL;
@@ -1706,7 +1818,7 @@ static const char *cmd_pdf_protect_method(cmd_parms *cmd, void *_dcfg,
 /* -- Geo Lookup configuration -- */
 
 static const char *cmd_geo_lookup_db(cmd_parms *cmd, void *_dcfg,
-    const char *p1)
+                                     const char *p1)
 {
     const char *filename = resolve_relative_path(cmd->pool, cmd->directive->filename, p1);
     char *error_msg;
@@ -1723,7 +1835,9 @@ static const char *cmd_geo_lookup_db(cmd_parms *cmd, void *_dcfg,
 
 /* -- Cache -- */
 
-static const char *cmd_cache_transformations(cmd_parms *cmd, void *_dcfg, const char *p1, const char *p2) {
+static const char *cmd_cache_transformations(cmd_parms *cmd, void *_dcfg,
+                                             const char *p1, const char *p2)
+{
     directory_config *dcfg = (directory_config *)_dcfg;
 
     if (dcfg == NULL) return NULL;
@@ -2016,6 +2130,22 @@ const command_rec module_directives[] = {
         "marker for a skipAfter target"
     ),
 
+    AP_INIT_TAKE1 (
+        "SecPcreMatchLimit",
+        cmd_pcre_match_limit,
+        NULL,
+        CMD_SCOPE_MAIN,
+        "PCRE match limit"
+    ),
+
+    AP_INIT_TAKE1 (
+        "SecPcreMatchLimitRecursion",
+        cmd_pcre_match_limit_recursion,
+        NULL,
+        CMD_SCOPE_MAIN,
+        "PCRE match limit recursion"
+    ),
+
     AP_INIT_FLAG (
         "SecPdfProtect",
         cmd_pdf_protect,
@@ -2224,6 +2354,14 @@ const command_rec module_directives[] = {
         "path to the file upload area"
     ),
 
+    AP_INIT_TAKE1 (
+        "SecUploadFileLimit",
+        cmd_upload_file_limit,
+        NULL,
+        CMD_SCOPE_ANY,
+        "limit the number of uploaded files processed"
+    ),
+
     AP_INIT_TAKE1 (
         "SecUploadFileMode",
         cmd_upload_filemode,
