@@ -1,6 +1,6 @@
 /*
  * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2009 Breach Security, Inc. (http://www.breach.com/)
+ * Copyright (c) 2004-2010 Breach Security, Inc. (http://www.breach.com/)
  *
  * This product is released under the terms of the General Public Licence,
  * version 2 (GPLv2). Please refer to the file LICENSE (included with this
@@ -93,10 +93,10 @@ typedef struct msc_string msc_string;
 #define REQUEST_BODY_FORCEBUF_OFF               0
 #define REQUEST_BODY_FORCEBUF_ON                1
 
-#define SECACTION_TARGETS                       "REQUEST_URI"
+#define SECACTION_TARGETS                       "REMOTE_ADDR"
 #define SECACTION_ARGS                          "@unconditionalMatch"
 
-#define SECMARKER_TARGETS                       "REQUEST_URI"
+#define SECMARKER_TARGETS                       "REMOTE_ADDR"
 #define SECMARKER_ARGS                          "@noMatch"
 #define SECMARKER_BASE_ACTIONS                  "t:none,pass,id:"
 
@@ -127,6 +127,10 @@ extern module AP_MODULE_DECLARE_DATA security2_module;
 
 extern DSOLOCAL const command_rec module_directives[];
 
+extern DSOLOCAL unsigned long int msc_pcre_match_limit;
+
+extern DSOLOCAL unsigned long int msc_pcre_match_limit_recursion;
+
 #define RESBODY_STATUS_NOT_READ         0   /* we were not configured to read the body */
 #define RESBODY_STATUS_ERROR            1   /* error occured while we were reading the body */
 #define RESBODY_STATUS_PARTIAL          2   /* partial body content available in the brigade */
@@ -327,6 +331,7 @@ struct modsec_rec {
     /* upload */
     int                  upload_extract_files;
     int                  upload_remove_files;
+    int                  upload_files_count;
 
     /* other */
     apr_table_t         *collections_original;
@@ -433,6 +438,7 @@ struct directory_config {
     int                  upload_keep_files;
     int                  upload_validates_files;
     int                  upload_filemode; /* int only so NOT_SET works */
+    int                  upload_file_limit;
 
     /* Used only in the configuration phase. */
     msre_rule           *tmp_chain_starter;
